import static org.junit.jupiter.api.Assertions.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import greenfoot.junitUtils.jupiter.runner.GreenfootRunner;

import greenfoot.World;
import greenfoot.junitUtils.Random;
import greenfoot.junitUtils.WorldCreator;

/**
 * Test Driven Development (TDD)
 * 
 * @author Francisco Guerra
 */
@ExtendWith(GreenfootRunner.class)
class CrabTest {

    Crab crab;
    
    @BeforeEach
    void setUp() throws Exception {
        // GIVEN a crab into any world
        World world = WorldCreator.getWorld(560, 560, 1);
        crab = new Crab();
        world.addObject(crab, 231, 203);
    }

    @Test
    void testGetX() throws Exception {
        // WHEN nothing
        
        // THEN the crab is still in the same place
        assertEquals(231, crab.getX());
    }

    @Test
    void testGetY() throws Exception {
        // WHEN nothing
        
        // THEN the crab is still in the same place
        assertEquals(203, crab.getY());
    }
    
    @Test
    void testGetRotation() throws Exception {
        // WHEN nothing
        
        // THEN the crab is still in the same direction
        assertEquals(0, crab.getRotation());
    }
    
    @Test
    void testGetImage() throws Exception {
        // WHEN nothing
        
        // THEN the crab has its initial image
        assertEquals("crab.png", crab.getImage().getImageFileName());
    }
    
    @Test
    void testMove() throws Exception {
        // GIVEN the initial position
        int x = crab.getX();
        int y = crab.getY();
        
        // WHEN the random number is greater than or equal to 10
        Random.set(10);
        crab.act();
        
        // THEN the crab moves 5
        assertEquals(x+5, crab.getX());
        assertEquals(  y, crab.getY());
    }
    
    @Test
    void testMoveAtEdge() throws Exception {
        // GIVEN the edge position
        int x = crab.getWorld().getWidth()-1;
        int y = 100;
        crab.setLocation(x, y);
        
        
        // WHEN the random number is greater than or equal to 10
        Random.set(10);
        crab.act();
        
        // THEN the crab moves 1 and turns 17
        assertEquals(  x, crab.getX());
        assertEquals(y+1, crab.getY());
        assertEquals( 17, crab.getRotation());
    }
    
    @Test
    void testTwoMoveAtEdge() throws Exception {
        // GIVEN the edge position
        int x = crab.getWorld().getWidth()-1;
        int y = 100;
        crab.setLocation(x, y);
        
        
        // WHEN the random number is greater than or equal to 10
        Random.set(10);
        crab.act();
        Random.set(10);
        crab.act();
        
        // THEN the crab moves 4 and turns 17+17
        assertEquals(  x, crab.getX());
        assertEquals(y+4, crab.getY());
        assertEquals( 34, crab.getRotation());
    }
    
    @Test
    void testTurnRight() throws Exception {
        // GIVEN the initial position of the crab
        int x = crab.getX();
        int y = crab.getY();
        
        // WHEN the random number is less than 10
        Random.set(9);  // random number is less than 10
        Random.set(90); // random number to turn 45 = 90 - 45
        crab.act();
        
        // THEN the crab turns 45 degrees and moves diagonally
        assertEquals(x+4, crab.getX());
        assertEquals(y+4, crab.getY());
        assertEquals( 45, crab.getRotation());
    }

    @Test
    void testTurnLeft() throws Exception {
        // GIVEN the initial position of the crab
        int x = crab.getX();
        int y = crab.getY();
        
        // WHEN the random number is less than 10
        Random.set(9); // random number is less than 10
        Random.set(0); // random number to turn -45 = 0 - 45
        crab.act();
        
        // THEN the crab turns -45 degrees and moves diagonally
        assertEquals(x+4, crab.getX());
        assertEquals(y-4, crab.getY());
        assertEquals(315, crab.getRotation());
    }

    @Test
    void testTurnRightAtEdge() throws Exception {
        // GIVEN the edge position
        int x = crab.getWorld().getWidth()-1;
        int y = 100;
        crab.setLocation(x, y);
        
        
        // WHEN the random number is less than 10
        Random.set(9);  // random number is less than 10
        Random.set(90); // random number to turn 45 = 90 - 45
        crab.act();
        
        // THEN the crab moves 4 and turns 62
        assertEquals(    x, crab.getX());
        assertEquals(  y+4, crab.getY());
        assertEquals(45+17, crab.getRotation());
    }
    
    @Test
    void testTurnLeftAtEdge() throws Exception {
        // GIVEN the edge position
        int x = crab.getWorld().getWidth()-1;
        int y = 100;
        crab.setLocation(x, y);
        
        
        // WHEN the random number is less than 10
        Random.set(9); // random number is less than 10
        Random.set(0); // random number to turn -45 = 0 - 45
        crab.act();
        
        // THEN the crab moves -2 and turns -28
        assertEquals(        x, crab.getX());
        assertEquals(      y-2, crab.getY());
        assertEquals(360+17-45, crab.getRotation());
    }
    
}
